/**
SerIO Triple Axis Accelerometer Sensor Example Application
Written by Ryan Owens
SparkFun Electronics

Sketch Description:
This sketch illustrates how to interface between the SerIO platform from SparkFun Electronics
and the ADXL335 Triple Axis acceleromter. The sketch draws a 3d cube and manipulates it's position based on the orientation of the accelerometer.

Circuit Description:
Z axis to pin A0
Y axis to pin A1
X axis to pin A2
Vcc connected to 3.3V
Gnd connected to GND.
 */

//The Processing Serial Library will be used to communicate with the SerIO.
import processing.serial.*;

//Create variables so the sketch knows which SerIO pins each axis is connected to.
int Xaccel=2;
int Yaccel=1;
int Zaccel=0;
int NUM_SENSORS=3;

Serial myPort;  // Create object from Serial class
String response=""; //Create a string to hold the responses from the SerIO board.

int[] sensor_positions = {Xaccel, Yaccel, Zaccel};

int[] a_values = new int[3];  //Create an array to hold the analog values read from the SerIO
int[] voltages = new int[3];  //An array to hold the voltages which have been converted from analog values
float[] g_force = new float[3]; //An array to hold the 'G' value of each axis.
float[] normal_vectors = new float[3]; //An array to hold the normalized values of the 'G' array.
String[] search;             //Used while parsing the response string from the SerIO for values
float Axz=0, Ayz=0;          //These will hold the X-Z and Y-Z angles of the accelerometer.

void setup() 
{
  size(640, 360, P3D);   //Set up the sketch window.
  noStroke(); 
  colorMode(RGB, 1);   
  
  //Set up the serial port. You may need to change this to reflect your systems COM ports.
  String portName = Serial.list()[1];
  myPort = new Serial(this, portName, 57600);
  
  //Wait for the SerIO to issues the 'Ready' command.
  while(match(getResponse(), "Ready") == null);
}

void draw() {
  background(255);
  
  for(int cur_sensor=0; cur_sensor<NUM_SENSORS; cur_sensor++)
  {
    //Send command to read current analog pin
    myPort.write("a,"+str(sensor_positions[cur_sensor])+"\r");
    
    //Get the response string from serIO
    response=getResponse();
    //println(response);
    
    //Find out if this the response contains a value
    search = match(response, "Value");
    //If we found the word 'Value' in the response, parse out the integer value.
    if(search != null){
      String[] list = split(response, ": ");  //Split the response into 3 sections. The actual value will be in the last secion
      a_values[cur_sensor]=int(parseFloat(list[2]));  //Convert serial value into an integer in the range 0-255 (multiply 0-1023 by .249 to get number in 0-255)
    }    
  }
  
  for(int i=0; i<NUM_SENSORS; i++)
  {
      voltages[i] = convertToVoltage(a_values[i]); //Convert analog reading to a voltage
      g_force[i] = convertToG(voltages[i]); //Convert voltage to a corresponding G value
  }    
  normal_vectors = normalize_array(g_force);  //Normalize the 'G' array. This ensures that the greatest value on any axis is 1.

  Axz=convertToDegrees(normal_vectors[2], normal_vectors[1]);  //Find the angle on the X-Z plane
  Ayz=convertToDegrees(normal_vectors[2], normal_vectors[0]);  //Find the angle on the Y-Z plane
  
  //Check the quadrant of the Z axis reading and change the angle accordingly.
  if(g_force[2] <0){
    Axz+=360;
    Ayz+=360;
  }

  //println(Axz);  //Uncomment this line to see the calculated X-Y angle.
  //println(Ayz); //Uncomment this line to see the calculated Y-Z angle
  
  //Draw the cube!
  pushMatrix(); 
  translate(width/2, height/2, -30); 
  
  //Rotate the cube based on the calculated angles from the accelerometer.
  rotateX(radians(Axz));
  rotateY(radians(Ayz));
  
  scale(90);
  beginShape(QUADS);

  fill(0, 1, 1); vertex(-1,  1,  1);
  fill(1, 1, 1); vertex( 1,  1,  1);
  fill(1, 0, 1); vertex( 1, -1,  1);
  fill(0, 0, 1); vertex(-1, -1,  1);

  fill(1, 1, 1); vertex( 1,  1,  1);
  fill(1, 1, 0); vertex( 1,  1, -1);
  fill(1, 0, 0); vertex( 1, -1, -1);
  fill(1, 0, 1); vertex( 1, -1,  1);

  fill(1, 1, 0); vertex( 1,  1, -1);
  fill(0, 1, 0); vertex(-1,  1, -1);
  fill(0, 0, 0); vertex(-1, -1, -1);
  fill(1, 0, 0); vertex( 1, -1, -1);

  fill(0, 1, 0); vertex(-1,  1, -1);
  fill(0, 1, 1); vertex(-1,  1,  1);
  fill(0, 0, 1); vertex(-1, -1,  1);
  fill(0, 0, 0); vertex(-1, -1, -1);

  fill(0, 1, 0); vertex(-1,  1, -1);
  fill(1, 1, 0); vertex( 1,  1, -1);
  fill(1, 1, 1); vertex( 1,  1,  1);
  fill(0, 1, 1); vertex(-1,  1,  1);

  fill(0, 0, 0); vertex(-1, -1, -1);
  fill(1, 0, 0); vertex( 1, -1, -1);
  fill(1, 0, 1); vertex( 1, -1,  1);
  fill(0, 0, 1); vertex(-1, -1,  1);

  endShape();
  
  popMatrix();  
  
}

//The getResponse function is used to get a response string from the SerIO platform.
//Function assumes that a global serial port has been created and opened and given the name myPort.
String getResponse()
{
  String serial_string="";  //This string will hold the characters received from the SerIO.
  int message_complete=0;  //Used as a flag to indicate the end of the response string.
  char val = 0;        // Data received from the serial port

  //Read characters from the Serial Port until the carriage return character is received.
  while(message_complete==0)
  {
    if(myPort.available() > 0)  //If a character is at the serial port, read it!
    {
      val=(char)myPort.read();  //Copy the character from the Serial Port.
      serial_string = serial_string + str(val);  //Add the newest character to the end of the string.
      if(val == '\r')  //If we receive a carriage return character, set the message_complete flag so we can exit the function
      {
        message_complete=1;
      }
    }
  }
  
  return serial_string;  //Send the response string back to the caller.
} 

//This function takes an adc value (retrieved from the SerIO 'Read Analog' function) and converts into millivolts.
int convertToVoltage(int adc_value)
{
  int voltage=0;
  
  voltage=(adc_value * 3300)/1024;  //Convert the count to millivolts
  
  return voltage;
}

//This function takes a voltage input in millivolts and converts it to a 'G' value according to the ADXL335 datasheet.
float convertToG(int voltage)
{
  float g_value=0.0;
  g_value = voltage-1650;  //Subtract the offset of the Zero G voltage
  g_value = g_value/330;  //Divide the voltage difference by 330mv/G
  
  return g_value;
}

//Normalizes a 3 position array
float[] normalize_array(float[] value)
{
  float NormValue=0;
  float[] normalized_array = new float[3];
  
  NormValue=sqrt(sq(value[0])+sq(value[1])+sq(value[2]));
  normalized_array[0]=value[0]/NormValue;
  normalized_array[1]=value[1]/NormValue;
  normalized_array[2]=value[2]/NormValue;
  
  return normalized_array;
}

//Finds the angle between two gravity vectors.
float convertToDegrees(float Rz, float Rx)
{
   float degree=0;
   degree=atan2(radians(Rz), radians(Rx));
   degree=degrees(degree);
   
   return degree;
}

